@@ -6,15 +6,20 @@ from __future__ import absolute_import, division, print_function
 __all__ = ["HtmlWriter"]
 
 import base64
-import cgi
 import io
 import logging
 import pkg_resources
 import string
 from PIL import Image
+import six
+if not six.PY2:
+    from html import escape as html_escape
+else:
+    from cgi import escape as html_escape
 
 from ged4py import model
-from .plotter import Plotter
+from .ancestor_tree import AncestorTree
+from .ancestor_tree_svg import SVGTreeVisitor
 from .size import Size
 from . import utils
 from . import writer
@@ -120,10 +125,10 @@ class HtmlWriter(writer.Writer):
         for piece in utils.split_refs(text):
             if isinstance(piece, tuple):
                 xref, name = piece
-                result += u'<a href="#{0}">{1}</a>'.format(cgi.escape(xref),
-                                                           cgi.escape(name))
+                result += u'<a href="#{0}">{1}</a>'.format(html_escape(xref),
+                                                           html_escape(name))
             else:
-                result += cgi.escape(piece)
+                result += html_escape(piece)
         return result
 
     def _render_section(self, level, ref_id, title, newpage=False):
@@ -138,7 +143,7 @@ class HtmlWriter(writer.Writer):
         """
         self._toc += [(level, ref_id, title)]
         doc = [u'<h{0} id="{1}">{2}</h{0}>\n'.format(level, ref_id,
-                                                     cgi.escape(title))]
+                                                     html_escape(title))]
         for line in doc:
             self._output.write(line.encode('utf-8'))
 
@@ -182,36 +187,29 @@ class HtmlWriter(writer.Writer):
 
         if families:
             hdr = self._tr.tr(TR("Spouses and children"), person.sex)
-            doc += ['<h3>' + cgi.escape(hdr) + '</h3>\n']
+            doc += ['<h3>' + html_escape(hdr) + '</h3>\n']
             for family in families:
                 family = self._interpolate(family)
                 doc += ['<p>' + family + '</p>\n']
 
         if events:
             hdr = self._tr.tr(TR("Events and dates"))
-            doc += ['<h3>' + cgi.escape(hdr) + '</h3>\n']
+            doc += ['<h3>' + html_escape(hdr) + '</h3>\n']
             for date, facts in events:
                 facts = self._interpolate(facts)
-                doc += ['<p>' + cgi.escape(date) + ": " + facts +
+                doc += ['<p>' + html_escape(date) + ": " + facts +
                         '</p>\n']
 
         if notes:
             hdr = self._tr.tr(TR("Comments"))
-            doc += ['<h3>' + cgi.escape(hdr) + '</h3>\n']
+            doc += ['<h3>' + html_escape(hdr) + '</h3>\n']
             for note in notes:
                 note = self._interpolate(note)
                 doc += ['<p>' + note + '</p>\n']
 
         # plot ancestors tree
-        tree_svg = self._make_ancestor_tree(person)
-        if tree_svg:
-            hdr = self._tr.tr(TR("Ancestor tree"))
-            doc += ['<h3>' + cgi.escape(hdr) + '</h3>\n']
-            doc += ['<div class="centered">\n']
-            doc += [tree_svg]
-            doc += ['</div>\n']
-        else:
-            doc += ['<svg width="100%" height="1pt"/>\n']
+        doc += self._make_ancestor_tree(person)
+
         for line in doc:
             self._output.write(line.encode('utf-8'))
 
@@ -274,7 +272,7 @@ class HtmlWriter(writer.Writer):
         """Produce table of contents using info collected in _render_section().
         """
         section = self._tr.tr(TR("Table Of Contents"))
-        doc = [u'<h1>{0}</h1>\n'.format(cgi.escape(section))]
+        doc = [u'<h1>{0}</h1>\n'.format(html_escape(section))]
         lvl = 0
         for toclvl, tocid, text in self._toc:
             while lvl < toclvl:
@@ -344,9 +342,18 @@ class HtmlWriter(writer.Writer):
         :return: Image data (XML contents), bytes
         """
         width = self._page_width ^ 'px'
-        plotter = Plotter(width=width, gen_dist="12pt", font_size="9pt",
-                          fullxml=False, refs=True, max_gen=self._tree_width)
-        img = plotter.parent_tree(person, 'px')
+        tree = AncestorTree(person, max_gen=self._tree_width, width=width, gen_dist="12pt", font_size="9pt")
+        visitor = SVGTreeVisitor(units='px', fullxml=False)
+        tree.visit(visitor)
+        img = visitor.makeSVG(width=tree.width, height=tree.height)
+        doc = []
         if img is not None:
-            return img[0]
-        return None
+            tree_svg = img[0]
+            hdr = self._tr.tr(TR("Ancestor tree"))
+            doc += ['<h3>' + html_escape(hdr) + '</h3>\n']
+            doc += ['<div class="centered">\n']
+            doc += [tree_svg]
+            doc += ['</div>\n']
+        else:
+            doc += ['<svg width="100%" height="1pt"/>\n']
+        return doc
